@@ -12,6 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-__version__ = '0.1.0'
+__version__ = '0.2.0'
 
-tekton_api_version = 'tekton.dev/v1beta1'
+from ._client import TektonClient  # noqa F401
