@@ -12,9 +12,57 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import re
+
 from kfp import dsl
 
 
+def sanitize_k8s_name(name,
+                      allow_capital_underscore=False,
+                      allow_dot=False,
+                      allow_slash=False,
+                      max_length=63,
+                      suffix_space=0):
+    """From _make_kubernetes_name
+      sanitize_k8s_name cleans and converts the names in the workflow.
+
+    https://kubernetes.io/docs/concepts/overview/working-with-objects/names/
+    https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#syntax-and-character-set
+    https://github.com/kubernetes/kubernetes/blob/c369cf18/staging/src/k8s.io/apimachinery/pkg/util/validation/validation.go#L89
+
+    Args:
+      name: original name
+      allow_capital_underscore: whether to allow capital letter and underscore
+        in this name (i.e. for parameters)
+      allow_dot: whether to allow dots in this name (i.e. for labels)
+      allow_slash: whether to allow slash in this name (i.e. for label and annotation keys)
+      max_length: maximum length of K8s name, default: 63
+      suffix_space: number of characters reserved for a suffix to be appended
+
+    Returns:
+      sanitized name.
+    """
+    k8s_name = re.sub('[^-_./0-9A-Za-z]+', '-', name)
+
+    if not allow_capital_underscore:
+      k8s_name = re.sub('_', '-', k8s_name.lower())
+
+    if not allow_dot:
+      k8s_name = re.sub('[.]', '-', k8s_name)
+
+    if not allow_slash:
+      k8s_name = re.sub('[/]', '-', k8s_name)
+
+    # replace duplicate dashes, strip enclosing dashes
+    k8s_name = re.sub('-+', '-', k8s_name).strip('-')
+
+    # truncate if length exceeds max_length
+    max_length = max_length - suffix_space
+    k8s_name = k8s_name[:max_length].rstrip('-') if len(k8s_name) > max_length else k8s_name
+
+    return k8s_name
+
+
 def convert_k8s_obj_to_json(k8s_obj):
     """
     Builds a JSON K8s object.
@@ -39,9 +87,6 @@ def convert_k8s_obj_to_json(k8s_obj):
     if k8s_obj is None:
       return None
     elif isinstance(k8s_obj, PRIMITIVE_TYPES):
-      # if re.search(r'^{{.*}}$', k8s_obj):
-      #   k8s_obj = k8s_obj.replace('{{', '$(')
-      #   k8s_obj = k8s_obj.replace('}}', ')')
       return k8s_obj
     elif isinstance(k8s_obj, list):
       return [convert_k8s_obj_to_json(sub_obj)
@@ -51,7 +96,7 @@ def convert_k8s_obj_to_json(k8s_obj):
                    for sub_obj in k8s_obj)
     elif isinstance(k8s_obj, (datetime, date)):
       return k8s_obj.isoformat()
-    elif isinstance(k8s_obj, dsl.PipelineParam): 
+    elif isinstance(k8s_obj, dsl.PipelineParam):
       if isinstance(k8s_obj.value, str):
         return k8s_obj.value
       return '$(inputs.params.%s)' % k8s_obj.full_name  # change for Tekton
@@ -64,11 +109,9 @@ def convert_k8s_obj_to_json(k8s_obj):
       # and attributes which value is not None.
       # Convert attribute name to json key in
       # model definition for request.
-      attr_types = (k8s_obj.swagger_types if hasattr(k8s_obj, "swagger_types") 
-                    else k8s_obj.openapi_types)
       obj_dict = {k8s_obj.attribute_map[attr]: getattr(k8s_obj, attr)
-                  for attr, _ in iteritems(attr_types)
+                  for attr in k8s_obj.attribute_map
                   if getattr(k8s_obj, attr) is not None}
 
     return {key: convert_k8s_obj_to_json(val)
-            for key, val in iteritems(obj_dict)}
\ No newline at end of file
+            for key, val in iteritems(obj_dict)}
