@@ -47,19 +47,16 @@ def parse_arguments():
   parser.add_argument('--disable-telemetry',
                       action='store_true',
                       help='disable adding telemetry labels, default is enabled.')
-  parser.add_argument('--generate-pipelinerun',
+  parser.add_argument('--disable_artifacts',
                       action='store_true',
-                      help='enable pipelinerun yaml generation')
-  parser.add_argument('--enable-artifacts',
-                      action='store_true',
-                      help='enable artifact inputs and outputs')
+                      help='disable artifact inputs and outputs')
 
   args = parser.parse_args()
   return args
 
 
-def _compile_pipeline_function(pipeline_funcs, function_name, output_path, type_check, allow_telemetry,
-                               generate_pipelinerun=False, enable_artifacts=False):
+def _compile_pipeline_function(pipeline_funcs, function_name, output_path, type_check,
+                               allow_telemetry, enable_artifacts=False):
   if len(pipeline_funcs) == 0:
     raise ValueError('A function with @dsl.pipeline decorator is required in the py file.')
 
@@ -75,20 +72,20 @@ def _compile_pipeline_function(pipeline_funcs, function_name, output_path, type_
   else:
     pipeline_func = pipeline_funcs[0]
 
-  TektonCompiler().compile(pipeline_func, output_path, type_check, allow_telemetry=allow_telemetry,
-                           generate_pipelinerun=generate_pipelinerun,
+  TektonCompiler().compile(pipeline_func, output_path, type_check,
+                           allow_telemetry=allow_telemetry,
                            enable_artifacts=enable_artifacts)
 
 
 def compile_pyfile(pyfile, function_name, output_path, type_check, allow_telemetry,
-                   generate_pipelinerun=False, enable_artifacts=False):
+                   enable_artifacts=False):
   sys.path.insert(0, os.path.dirname(pyfile))
   try:
     filename = os.path.basename(pyfile)
     with kfp_compiler_main.PipelineCollectorContext() as pipeline_funcs:
       __import__(os.path.splitext(filename)[0])
     _compile_pipeline_function(pipeline_funcs, function_name, output_path, type_check,
-                               allow_telemetry=allow_telemetry, generate_pipelinerun=generate_pipelinerun,
+                               allow_telemetry=allow_telemetry,
                                enable_artifacts=enable_artifacts)
   finally:
     del sys.path[0]
@@ -102,8 +99,9 @@ def main():
         raise ValueError('Either --py or --package is needed but not both.')
     if args.py:
         compile_pyfile(args.py, args.function, args.output, not args.disable_type_check, not args.disable_telemetry,
-                       args.generate_pipelinerun, args.enable_artifacts)
+                       not args.disable_artifacts)
     else:
         if args.namespace is None:
             raise ValueError('--namespace is required for compiling packages.')
-            kfp_compiler_main.compile_package(args.package, args.namespace, args.function, args.output, not args.disable_type_check)
+        # TODO: deprecated, but without the monkey-patch this will produce Argo YAML
+        kfp_compiler_main.compile_package(args.package, args.namespace, args.function, args.output, not args.disable_type_check)
